@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import asyncio
 import itertools
+import threading
 import uuid
 from dataclasses import dataclass, field
 from enum import Enum
@@ -30,6 +31,7 @@ from ..lsp_types import (
     ErrorCodes,
     PreviousResultId,
     ProgressToken,
+    PublishDiagnosticsParams,
     RelatedFullDocumentDiagnosticReport,
     RelatedUnchangedDocumentDiagnosticReport,
     ServerCapabilities,
@@ -74,10 +76,34 @@ class WorkspaceDocumentsResult:
 
 @dataclass
 class DiagnosticsData:
-    id: str
+    id: str = field(default_factory=lambda: str(uuid.uuid4()))
     entries: Dict[Any, Optional[List[Diagnostic]]] = field(default_factory=dict)
     version: Optional[int] = None
     task: Optional[asyncio.Task[Any]] = None
+    force: bool = False
+
+
+def _cancel_all_tasks(loop: asyncio.AbstractEventLoop) -> None:
+    to_cancel = asyncio.all_tasks(loop)
+    if not to_cancel:
+        return
+
+    for task in to_cancel:
+        task.cancel()
+
+    loop.run_until_complete(asyncio.gather(*to_cancel, loop=loop, return_exceptions=True))
+
+    for task in to_cancel:
+        if task.cancelled():
+            continue
+        if task.exception() is not None:
+            loop.call_exception_handler(
+                {
+                    "message": "unhandled exception during asyncio.run() shutdown",
+                    "exception": task.exception(),
+                    "task": task,
+                }
+            )
 
 
 class DiagnosticsProtocolPart(LanguageServerProtocolPart, HasExtendCapabilities):
@@ -97,18 +123,65 @@ class DiagnosticsProtocolPart(LanguageServerProtocolPart, HasExtendCapabilities)
 
         self._current_workspace_task: Optional[asyncio.Task[WorkspaceDiagnosticReport]] = None
 
+        self._diagnostics_loop: Optional[asyncio.AbstractEventLoop] = None
+        self._diagnostics_loop_lock = threading.RLock()
+        self._diagnostics_started = threading.Event()
+
+        self.parent.on_initialized.add(self.initialized)
+
+        self.parent.documents.did_close.add(self.on_did_close)
+
         self.in_get_workspace_diagnostics = Event(True)
 
+    async def initialized(self, sender: Any) -> None:
+        self._ensure_diagnostics_thread_started()
+
+    @property
+    def diagnostics_loop(self) -> asyncio.AbstractEventLoop:
+        if self._diagnostics_loop is None:
+            self._ensure_diagnostics_thread_started()
+
+        assert self._diagnostics_loop is not None
+
+        return self._diagnostics_loop
+
+    def _run_diagnostics(self) -> None:
+        loop = asyncio.new_event_loop()
+        asyncio.set_event_loop(loop)
+        try:
+            self._diagnostics_loop = loop
+            self._diagnostics_started.set()
+
+            loop.slow_callback_duration = 10
+
+            loop.run_forever()
+            _cancel_all_tasks(loop)
+            loop.run_until_complete(loop.shutdown_asyncgens())
+        finally:
+            asyncio.set_event_loop(None)
+            loop.close()
+
+    def _ensure_diagnostics_thread_started(self) -> None:
+        with self._diagnostics_loop_lock:
+            if self._diagnostics_loop is None:
+                self._server_thread = threading.Thread(
+                    name="diagnostics_worker", target=self._run_diagnostics, daemon=True
+                )
+
+                self._server_thread.start()
+
+                if not self._diagnostics_started.wait(10):
+                    raise RuntimeError("Can't start diagnostics worker thread.")
+
     def extend_capabilities(self, capabilities: ServerCapabilities) -> None:
         if (
             self.parent.client_capabilities is not None
             and self.parent.client_capabilities.text_document is not None
             and self.parent.client_capabilities.text_document.diagnostic is not None
         ):
-            # capabilities.diagnostic_provider = None
             capabilities.diagnostic_provider = DiagnosticOptions(
                 inter_file_dependencies=True,
-                workspace_diagnostics=True,
+                workspace_diagnostics=False,
                 identifier=f"robotcodelsp_{uuid.uuid4()}",
                 work_done_progress=True,
             )
@@ -117,19 +190,6 @@ class DiagnosticsProtocolPart(LanguageServerProtocolPart, HasExtendCapabilities)
     async def collect(sender, document: TextDocument) -> DiagnosticsResult:  # NOSONAR
         ...
 
-    @async_tasking_event_iterator
-    async def collect_document_has_diagnostics(sender, document: TextDocument) -> bool:  # NOSONAR
-        ...
-
-    async def document_has_diagnostics(self, document: TextDocument) -> bool:  # NOSONAR
-        async for result in self.collect_document_has_diagnostics(
-            self, document, callback_filter=language_id_filter(document)
-        ):
-            if result:
-                return True
-
-        return False
-
     @async_tasking_event
     async def load_workspace_documents(sender) -> List[WorkspaceDocumentsResult]:  # NOSONAR
         ...
@@ -167,10 +227,54 @@ class DiagnosticsProtocolPart(LanguageServerProtocolPart, HasExtendCapabilities)
                         self._workspace_loaded = True
                         self.workspace_loaded_event.set()
                         await self.on_workspace_loaded(self)
-                        await self.refresh()
+                        await self.force_refresh_all()
+
+    async def force_refresh_all(self, refresh: bool = True) -> None:
+        for doc in self.parent.documents.documents:
+            self.get_diagnostics_data(doc).force = True
+
+        if refresh:
+            await self.refresh()
+
+    async def force_refresh_document(self, document: TextDocument, refresh: bool = True) -> None:
+        self.get_diagnostics_data(document).force = True
+        if refresh and document.opened_in_editor:
+            await self.refresh()
 
     @_logger.call
-    async def _get_diagnostics(self, document: TextDocument, data: DiagnosticsData) -> None:
+    @threaded()
+    async def on_did_close(self, sender: Any, document: TextDocument) -> None:
+        if await self.get_diagnostics_mode(document.uri) == DiagnosticsMode.WORKSPACE:
+            return
+
+        try:
+            data = self.get_diagnostics_data(document)
+            if data.task is not None and not data.task.done():
+                self._logger.debug(lambda: f"try to cancel diagnostics for {document}")
+
+                e = threading.Event()
+
+                def done(t: asyncio.Task[Any]) -> None:
+                    e.set()
+
+                data.task.add_done_callback(done)
+                data.task.get_loop().call_soon_threadsafe(data.task.cancel)
+
+                e.wait(120)
+
+        finally:
+            self.parent.send_notification(
+                "textDocument/publishDiagnostics",
+                PublishDiagnosticsParams(
+                    uri=document.document_uri,
+                    version=document._version,
+                    diagnostics=[],
+                ),
+            )
+
+    @_logger.call
+    async def _get_diagnostics_for_document(self, document: TextDocument, data: DiagnosticsData) -> None:
+        self._logger.debug(lambda: f"Get diagnostics for {document}")
 
         await asyncio.sleep(0.75)
 
@@ -193,12 +297,23 @@ class DiagnosticsProtocolPart(LanguageServerProtocolPart, HasExtendCapabilities)
                     if result.diagnostics is not None:
                         collected_keys.append(result.key)
 
-                    await self.refresh()
+                    # if document.opened_in_editor:
+                    #     await self.refresh()
+
+                    if data.entries:
+                        self.parent.send_notification(
+                            "textDocument/publishDiagnostics",
+                            PublishDiagnosticsParams(
+                                uri=document.document_uri,
+                                version=document._version,
+                                diagnostics=[
+                                    e for e in itertools.chain(*(i for i in data.entries.values() if i is not None))
+                                ],
+                            ),
+                        )
 
         except asyncio.CancelledError:
-            self._logger.critical(lambda: f"_get_diagnostics cancelled for {document}")
-        else:
-            await self.refresh()
+            self._logger.debug(lambda: f"_get_diagnostics cancelled for {document}")
         finally:
             for k in set(data.entries.keys()) - set(collected_keys):
                 data.entries.pop(k)
@@ -214,63 +329,61 @@ class DiagnosticsProtocolPart(LanguageServerProtocolPart, HasExtendCapabilities)
         **kwargs: Any,
     ) -> DocumentDiagnosticReport:
         try:
-            # if not self.workspace_loaded_event.is_set():
-            #     raise JsonRPCErrorException(
-            #         ErrorCodes.SERVER_CANCELLED,
-            #         "Workspace not loaded.",
-            #         data=DiagnosticServerCancellationData(True),
-            #     )
-
             document = await self.parent.documents.get(text_document.uri)
             if document is None:
-                raise JsonRPCErrorException(ErrorCodes.INVALID_PARAMS, f"Document {text_document!r} not found.")
+                raise JsonRPCErrorException(ErrorCodes.SERVER_CANCELLED, f"Document {text_document!r} not found.")
 
-            data: DiagnosticsData = document.get_data(self, None)
+            data = self.get_diagnostics_data(document)
 
-            if data is None:
-                data = DiagnosticsData(str(uuid.uuid4()))
-                document.set_data(self, data)
-
-            if (
-                document.version != data.version
-                or data.task is None
-                or not await self.document_has_diagnostics(document)
-            ):
+            if data.force or document.version != data.version or data.task is None:
 
                 task = data.task
 
-                data = DiagnosticsData(str(uuid.uuid4()))
-                document.set_data(self, data)
+                data.force = False
 
                 if task is not None and not task.done():
-                    self._logger.critical(lambda: f"try to cancel diagnostics for {document}")
+                    self._logger.debug(lambda: f"try to cancel diagnostics for {document}")
                     task.get_loop().call_soon_threadsafe(task.cancel)
 
                 data.version = document.version
                 data.task = create_sub_task(
-                    self._get_diagnostics(document, data), loop=self.parent.loop, name=f"diagnostics ${text_document}"
+                    self._get_diagnostics_for_document(document, data),
+                    loop=self.diagnostics_loop,
+                    name=f"diagnostics ${text_document}",
                 )
 
                 def done(t: asyncio.Task[Any]) -> None:
-                    if t.cancelled():
-                        self._logger.critical(lambda: f"diagnostics for {document} canceled")
-                        try:
-                            t.exception()
-                        except asyncio.CancelledError:
-                            pass
+
+                    self._logger.debug(lambda: f"diagnostics for {document} {'canceled' if t.cancelled() else 'ended'}")
+                    if t.done() and not t.cancelled():
+                        ex = t.exception()
+
+                        if ex is None or isinstance(ex, asyncio.CancelledError):
+                            return
 
                 data.task.add_done_callback(done)
 
             if data.id == previous_result_id:
                 return RelatedUnchangedDocumentDiagnosticReport(result_id=data.id)
 
-            return RelatedFullDocumentDiagnosticReport(
-                list(itertools.chain(*(e for e in data.entries.values() if e is not None))), result_id=data.id
-            )
+            # return RelatedFullDocumentDiagnosticReport(
+            #     list(itertools.chain(*(e for e in data.entries.values() if e is not None))), result_id=data.id
+            # )
+
+            return RelatedFullDocumentDiagnosticReport([], result_id=data.id)
         except asyncio.CancelledError:
-            self._logger.critical("canceled _text_document_diagnostic")
+            self._logger.debug("canceled _text_document_diagnostic")
             raise
 
+    def get_diagnostics_data(self, document: TextDocument) -> DiagnosticsData:
+        data: DiagnosticsData = document.get_data(self, None)
+
+        if data is None:
+            data = DiagnosticsData(str(uuid.uuid4()))
+            document.set_data(self, data)
+
+        return data
+
     @rpc_method(name="workspace/diagnostic", param_type=WorkspaceDiagnosticParams)
     @threaded()
     async def _workspace_diagnostic(
