@@ -360,13 +360,17 @@ class _ResourcesEntry(_ImportEntry):
         return self._document
 
     async def get_namespace(self) -> Namespace:
-        return await self.parent.parent_protocol.documents_cache.get_resource_namespace(await self.get_document())
+        async with self._lock:
+            return await self._get_namespace()
+
+    async def _get_namespace(self) -> Namespace:
+        return await self.parent.parent_protocol.documents_cache.get_resource_namespace(await self._get_document())
 
     async def get_libdoc(self) -> LibraryDoc:
         if self._lib_doc is None:
             async with self._lock:
                 if self._lib_doc is None:
-                    self._lib_doc = await (await self.get_namespace()).get_library_doc()
+                    self._lib_doc = await (await self._get_namespace()).get_library_doc()
         return self._lib_doc
 
 
@@ -705,20 +709,14 @@ class ImportsManager:
         from robot.variables.search import contains_variable
 
         if contains_variable(name, "$@&%"):
-            return await asyncio.wait_for(
-                asyncio.get_running_loop().run_in_executor(
-                    # self.process_pool,
-                    None,
-                    find_library,
-                    name,
-                    str(self.folder.to_path()),
-                    base_dir,
-                    self.config.python_path if self.config is not None else None,
-                    self.config.env if self.config is not None else None,
-                    self.config.variables if self.config is not None else None,
-                    variables,
-                ),
-                FIND_FILE_TIME_OUT,
+            return find_library(
+                name,
+                str(self.folder.to_path()),
+                base_dir,
+                self.config.python_path if self.config is not None else None,
+                self.config.env if self.config is not None else None,
+                self.config.variables if self.config is not None else None,
+                variables,
             )
 
         if name in STDLIBS:
@@ -743,21 +741,15 @@ class ImportsManager:
         from robot.variables.search import contains_variable
 
         if contains_variable(name, "$@&%"):
-            return await asyncio.wait_for(
-                asyncio.get_running_loop().run_in_executor(
-                    # self.process_pool,
-                    None,
-                    find_file,
-                    name,
-                    str(self.folder.to_path()),
-                    base_dir,
-                    self.config.python_path if self.config is not None else None,
-                    self.config.env if self.config is not None else None,
-                    self.config.variables if self.config is not None else None,
-                    variables,
-                    file_type,
-                ),
-                FIND_FILE_TIME_OUT,
+            return find_file(
+                name,
+                str(self.folder.to_path()),
+                base_dir,
+                self.config.python_path if self.config is not None else None,
+                self.config.env if self.config is not None else None,
+                self.config.variables if self.config is not None else None,
+                variables,
+                file_type,
             )
 
         return str(find_file_ex(name, base_dir, self.python_path, file_type))
@@ -770,20 +762,14 @@ class ImportsManager:
         from robot.variables.search import contains_variable
 
         if contains_variable(name, "$@&%"):
-            return await asyncio.wait_for(
-                asyncio.get_running_loop().run_in_executor(
-                    # self.process_pool,
-                    None,
-                    find_variables,
-                    name,
-                    str(self.folder.to_path()),
-                    base_dir,
-                    self.config.python_path if self.config is not None else None,
-                    self.config.env if self.config is not None else None,
-                    self.config.variables if self.config is not None else None,
-                    variables,
-                ),
-                FIND_FILE_TIME_OUT,
+            return find_variables(
+                name,
+                str(self.folder.to_path()),
+                base_dir,
+                self.config.python_path if self.config is not None else None,
+                self.config.env if self.config is not None else None,
+                self.config.variables if self.config is not None else None,
+                variables,
             )
 
         if get_robot_version() >= (5, 0):
@@ -963,6 +949,7 @@ class ImportsManager:
                     end_line_no=-1,
                     libname=libdoc.name,
                     libtype=libdoc.type,
+                    longname=f"{libdoc.name}.{kw[0].name}",
                     is_embedded=is_embedded_keyword(kw[0].name),
                     errors=getattr(kw[1], "errors") if hasattr(kw[1], "errors") else None,
                     is_error_handler=isinstance(kw[1], UserErrorHandler),
@@ -970,6 +957,7 @@ class ImportsManager:
                     if isinstance(kw[1], UserErrorHandler)
                     else None,
                     arguments=ArgumentSpec.from_robot_argument_spec(kw[1].arguments),
+                    parent=libdoc,
                 )
                 for kw in [(KeywordDocBuilder(resource=True).build_keyword(lw), lw) for lw in lib.handlers]
             ],
@@ -1100,84 +1088,53 @@ class ImportsManager:
     async def complete_library_import(
         self, name: Optional[str], base_dir: str = ".", variables: Optional[Dict[str, Any]] = None
     ) -> Optional[List[CompleteResult]]:
-        result = await asyncio.wait_for(
-            asyncio.get_running_loop().run_in_executor(
-                # self.process_pool,
-                None,
-                complete_library_import,
-                name,
-                str(self.folder.to_path()),
-                base_dir,
-                self.config.python_path if self.config is not None else None,
-                self.config.env if self.config is not None else None,
-                self.config.variables if self.config is not None else None,
-                variables,
-            ),
-            COMPLETE_LIBRARY_IMPORT_TIME_OUT,
+        return complete_library_import(
+            name,
+            str(self.folder.to_path()),
+            base_dir,
+            self.config.python_path if self.config is not None else None,
+            self.config.env if self.config is not None else None,
+            self.config.variables if self.config is not None else None,
+            variables,
         )
 
-        return result
-
     async def complete_resource_import(
         self, name: Optional[str], base_dir: str = ".", variables: Optional[Dict[str, Any]] = None
     ) -> Optional[List[CompleteResult]]:
-        result = await asyncio.wait_for(
-            asyncio.get_running_loop().run_in_executor(
-                # self.process_pool,
-                None,
-                complete_resource_import,
-                name,
-                str(self.folder.to_path()),
-                base_dir,
-                self.config.python_path if self.config is not None else None,
-                self.config.env if self.config is not None else None,
-                self.config.variables if self.config is not None else None,
-                variables,
-            ),
-            COMPLETE_RESOURCE_IMPORT_TIME_OUT,
+        return complete_resource_import(
+            name,
+            str(self.folder.to_path()),
+            base_dir,
+            self.config.python_path if self.config is not None else None,
+            self.config.env if self.config is not None else None,
+            self.config.variables if self.config is not None else None,
+            variables,
         )
 
-        return result
-
     async def complete_variables_import(
         self, name: Optional[str], base_dir: str = ".", variables: Optional[Dict[str, Any]] = None
     ) -> Optional[List[CompleteResult]]:
-        result = await asyncio.wait_for(
-            asyncio.get_running_loop().run_in_executor(
-                # self.process_pool,
-                None,
-                complete_variables_import,
-                name,
-                str(self.folder.to_path()),
-                base_dir,
-                self.config.python_path if self.config is not None else None,
-                self.config.env if self.config is not None else None,
-                self.config.variables if self.config is not None else None,
-                variables,
-            ),
-            COMPLETE_VARIABLES_IMPORT_TIME_OUT,
-        )
 
-        return result
+        return complete_variables_import(
+            name,
+            str(self.folder.to_path()),
+            base_dir,
+            self.config.python_path if self.config is not None else None,
+            self.config.env if self.config is not None else None,
+            self.config.variables if self.config is not None else None,
+            variables,
+        )
 
     async def resolve_variable(
         self, name: str, base_dir: str = ".", variables: Optional[Dict[str, Any]] = None, ignore_errors: bool = True
     ) -> Any:
-        result = await asyncio.wait_for(
-            asyncio.get_running_loop().run_in_executor(
-                # self.process_pool,
-                None,
-                resolve_variable,
-                name,
-                str(self.folder.to_path()),
-                base_dir,
-                self.config.python_path if self.config is not None else None,
-                self.config.env if self.config is not None else None,
-                self.config.variables if self.config is not None else None,
-                variables,
-                ignore_errors,
-            ),
-            COMPLETE_VARIABLES_IMPORT_TIME_OUT,
+        return resolve_variable(
+            name,
+            str(self.folder.to_path()),
+            base_dir,
+            self.config.python_path if self.config is not None else None,
+            self.config.env if self.config is not None else None,
+            self.config.variables if self.config is not None else None,
+            variables,
+            ignore_errors,
         )
-
-        return result
