@@ -432,13 +432,19 @@ class KeywordDoc(SourceEntity):
 
             result += "\n| | | | |"
             result += "\n|:--- | --:|:--|:---|"
-            bs = "\\"
+
             for a in self.args:
+                prefix = ""
+                if a.kind == KeywordArgumentKind.VAR_POSITIONAL:
+                    prefix = "*"
+                elif a.kind == KeywordArgumentKind.VAR_NAMED:
+                    prefix = "**"
+
                 result += (
-                    f"\n| {str(a.name)}"
+                    f"\n| `{prefix}{str(a.name)}`"
                     f"| {'=' if a.default_value is not None else ''}"
-                    f"| {str(a.default_value) if a.default_value is not None else ''}"
-                    f"| {' or '.join(f'{bs}<{s}>' for s in a.types) if a.types is not None else ''}|"
+                    f"| {f'`{str(a.default_value)}`' if a.default_value else ''}"
+                    f"| {' or '.join(f'`<{s}>`' for s in a.types) if a.types is not None else ''} |"
                 )
 
         return result
@@ -1127,6 +1133,11 @@ def get_robot_library_html_doc_str(name: str, working_dir: str = ".", base_dir:
 
     _update_env(working_dir)
 
+    if Path(name).suffix.lower() in ALLOWED_RESOURCE_FILE_EXTENSIONS:
+        name = find_file(name, working_dir, base_dir)
+    elif Path(name).suffix.lower() in ALLOWED_LIBRARY_FILE_EXTENSIONS:
+        name = find_file(name, working_dir, base_dir, file_type="Library")
+
     robot_libdoc = LibraryDocumentation(name)
     robot_libdoc.convert_docs_to_html()
     with io.StringIO() as output:
