@@ -4,25 +4,35 @@ import contextlib
 import socket
 import threading
 import traceback
+import urllib.parse
 from concurrent.futures import ProcessPoolExecutor
+from dataclasses import dataclass
 from http.server import SimpleHTTPRequestHandler, ThreadingHTTPServer
 from threading import Thread
-from typing import TYPE_CHECKING, Any, List, Optional, Union, cast
+from typing import TYPE_CHECKING, Any, List, Optional, Tuple, Union, cast
 from urllib.parse import parse_qs, urlparse
 
+from ....jsonrpc2.protocol import rpc_method
 from ....utils.logging import LoggingDescriptor
-from ....utils.net import check_free_port
+from ....utils.net import find_free_port
+from ....utils.uri import Uri
 from ...common.decorators import code_action_kinds, language_id
 from ...common.lsp_types import (
     CodeAction,
     CodeActionContext,
     CodeActionKinds,
     Command,
+    Model,
     Range,
 )
 from ...common.text_document import TextDocument
-from ..diagnostics.library_doc import get_library_doc, get_robot_library_html_doc_str
-from ..diagnostics.namespace import LibraryEntry
+from ..configuration import DocumentationServerConfig
+from ..diagnostics.library_doc import (
+    get_library_doc,
+    get_robot_library_html_doc_str,
+    resolve_robot_variables,
+)
+from ..diagnostics.namespace import LibraryEntry, Namespace
 from ..utils.ast_utils import Token, get_node_at_position
 from .model_helper import ModelHelperMixin
 
@@ -33,6 +43,12 @@ from string import Template
 
 from .protocol_part import RobotLanguageServerProtocolPart
 
+
+@dataclass(repr=False)
+class ConvertUriParams(Model):
+    uri: str
+
+
 HTML_ERROR_TEMPLATE = Template(
     """\n
 <!doctype html>
@@ -166,10 +182,10 @@ class RobotCodeActionProtocolPart(RobotLanguageServerProtocolPart, ModelHelperMi
 
         self._documentation_server: Optional[ThreadingHTTPServer] = None
         self._documentation_server_lock = threading.RLock()
-        self._documentation_server_port = check_free_port(3000)
+        self._documentation_server_port = 0
 
     async def initialized(self, sender: Any) -> None:
-        self._ensure_http_server_started()
+        await self._ensure_http_server_started()
 
     async def shutdown(self, sender: Any) -> None:
         with self._documentation_server_lock:
@@ -177,8 +193,13 @@ class RobotCodeActionProtocolPart(RobotLanguageServerProtocolPart, ModelHelperMi
                 self._documentation_server.shutdown()
                 self._documentation_server = None
 
-    def _run_server(self) -> None:
-        with DualStackServer(("", self._documentation_server_port), LibDocRequestHandler) as server:
+    def _run_server(self, start_port: int, end_port: int) -> None:
+
+        self._documentation_server_port = find_free_port(start_port, end_port)
+
+        self._logger.info(f"Start documentation server on port {self._documentation_server_port}")
+
+        with DualStackServer(("127.0.0.1", self._documentation_server_port), LibDocRequestHandler) as server:
             self._documentation_server = server
             try:
                 server.serve_forever()
@@ -186,10 +207,17 @@ class RobotCodeActionProtocolPart(RobotLanguageServerProtocolPart, ModelHelperMi
                 self._documentation_server = None
                 raise
 
-    def _ensure_http_server_started(self) -> None:
+    async def _ensure_http_server_started(self) -> None:
+        config = await self.parent.workspace.get_configuration(DocumentationServerConfig)
+
         with self._documentation_server_lock:
             if self._documentation_server is None:
-                self._server_thread = Thread(name="documentation_server", target=self._run_server, daemon=True)
+                self._server_thread = Thread(
+                    name="documentation_server",
+                    target=self._run_server,
+                    args=(config.start_port, config.end_port),
+                    daemon=True,
+                )
                 self._server_thread.start()
 
     @language_id("robotframework")
@@ -201,6 +229,7 @@ class RobotCodeActionProtocolPart(RobotLanguageServerProtocolPart, ModelHelperMi
 
         from robot.parsing.lexer import Token as RobotToken
         from robot.parsing.model.statements import (
+            Fixture,
             KeywordCall,
             LibraryImport,
             ResourceImport,
@@ -214,11 +243,14 @@ class RobotCodeActionProtocolPart(RobotLanguageServerProtocolPart, ModelHelperMi
 
         node = await get_node_at_position(model, range.start)
 
-        self._ensure_http_server_started()
+        await self._ensure_http_server_started()
 
         if isinstance(node, (LibraryImport, ResourceImport)):
 
-            args = f"&args={'::'.join(node.args)}" if isinstance(node, LibraryImport) and node.args else ""
+            url = await self.build_url(
+                node.name, node.args if isinstance(node, LibraryImport) else (), document, namespace
+            )
+
             return [
                 CodeAction(
                     "Open Documentation",
@@ -226,19 +258,15 @@ class RobotCodeActionProtocolPart(RobotLanguageServerProtocolPart, ModelHelperMi
                     command=Command(
                         "Open Documentation",
                         "robotcode.showDocumentation",
-                        [
-                            f"http://localhost:{self._documentation_server_port}/?name={node.name}"
-                            f"{args}"
-                            f"&basedir={document.uri.to_path().parent}"
-                        ],
+                        [url],
                     ),
                 )
             ]
 
-        if isinstance(node, (KeywordCall)):
+        if isinstance(node, (KeywordCall, Fixture)):
             result = await self.get_keyworddoc_and_token_from_position(
-                node.keyword,
-                cast(Token, node.get_token(RobotToken.KEYWORD)),
+                node.keyword if isinstance(node, KeywordCall) else node.name,
+                cast(Token, node.get_token(RobotToken.KEYWORD if isinstance(node, KeywordCall) else RobotToken.NAME)),
                 [cast(Token, t) for t in node.get_tokens(RobotToken.ARGUMENT)],
                 namespace,
                 range.start,
@@ -270,7 +298,8 @@ class RobotCodeActionProtocolPart(RobotLanguageServerProtocolPart, ModelHelperMi
                     if entry is None:
                         return None
 
-                    args = f"&args={'::'.join(entry.args)}" if entry.args else ""
+                    url = await self.build_url(entry.import_name, entry.args, document, namespace, kw_doc.name)
+
                     return [
                         CodeAction(
                             "Open Documentation",
@@ -278,15 +307,58 @@ class RobotCodeActionProtocolPart(RobotLanguageServerProtocolPart, ModelHelperMi
                             command=Command(
                                 "Open Documentation",
                                 "robotcode.showDocumentation",
-                                [
-                                    f"http://localhost:{self._documentation_server_port}"
-                                    f"/?name={entry.import_name}"
-                                    f"{args}"
-                                    f"&basedir={document.uri.to_path().parent}"
-                                    f"#{kw_doc.name}"
-                                ],
+                                [url],
                             ),
                         )
                     ]
 
         return None
+
+    async def build_url(
+        self,
+        name: str,
+        args: Tuple[Any, ...],
+        document: TextDocument,
+        namespace: Namespace,
+        target: Optional[str] = None,
+    ) -> str:
+
+        base_dir = str(document.uri.to_path().parent)
+
+        robot_variables = resolve_robot_variables(
+            str(namespace.imports_manager.folder.to_path()),
+            base_dir,
+            variables=await namespace.get_resolvable_variables(),
+        )
+        try:
+            name = robot_variables.replace_string(name.replace("\\", "\\\\"), ignore_errors=False)
+
+            args = tuple(robot_variables.replace_string(v.replace("\\", "\\\\"), ignore_errors=False) for v in args)
+
+        except (SystemExit, KeyboardInterrupt):
+            raise
+        except BaseException:
+            pass
+
+        url_args = "::".join(args) if args else ""
+
+        base_url = f"http://localhost:{self._documentation_server_port}"
+        params = urllib.parse.urlencode({"name": name, "args": url_args, "basedir": base_dir})
+
+        url = f"{base_url}" f"/?&{params}" f"{f'#{target}' if target else ''}"
+
+        return url
+
+    @rpc_method(name="robot/documentationServer/convertUri", param_type=ConvertUriParams)
+    @_logger.call
+    async def _convert_uri(self, uri: str, *args: Any, **kwargs: Any) -> Optional[str]:
+        real_uri = Uri(uri)
+
+        folder = self.parent.workspace.get_workspace_folder(real_uri)
+
+        if folder:
+            path = real_uri.to_path().relative_to(folder.uri.to_path())
+
+            return f"http://localhost:{self._documentation_server_port}/{path}"
+
+        return None
